<span style="line-height: 0; display: inline-block; border: 1px solid black;">
{%- for color in palette -%}
<span style="width: {{ width }}px; height: 20px; background-color: {{ color }}; display: inline-block;"></span>
{%- endfor -%}
</span>
